package emst.algorithm;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import emst.ds.Edge;
import emst.ds.Graph;
import emst.ds.Vertex;

public class Kruskal {

	public static List<Edge> mst(Graph graph) {
		
		List<Edge> a = new LinkedList<Edge>();
		
		// Create a disjoint set
		List<List<Vertex>> disjointSet = makeSet(graph.getVertices());
		
		
		// Sort the edges by weight in nondecreasing order
		List<Edge> edges = graph.getEdges();		
		Collections.sort(edges, new Comparator<Edge>(){

			@Override
			public int compare(Edge o1, Edge o2) {
				if (o1.getWeight() < o2.getWeight())
					return -1;
				if (o1.getWeight() > o2.getWeight())
					return 1;
				else return 0;
			}});
		
		for (Edge e : edges) {
			if (!sameSet(disjointSet, e.getV1(), e.getV2())) {				
				a.add(e);
				unionSubsets(disjointSet, e.getV1(), e.getV2());
			}
				
		}				
		
		return a;
	}
	
	private static List<List<Vertex>> makeSet(List<Vertex> vertices) {
		
		List<List<Vertex>> set = new ArrayList<List<Vertex>>();
		
		for (Vertex v : vertices) {
			List<Vertex> subset = new ArrayList<Vertex>();
			subset.add(v);
			set.add(subset);
		}
		
		return set;
	}
	
	private static boolean sameSet(List<List<Vertex>> disjointSet, Vertex u, Vertex v) {
		for (List<Vertex> subset : disjointSet) {
			if (subset.contains(u) && subset.contains(v))
				return true;
		}
		return false;
	}
	
	private static List<List<Vertex>> unionSubsets(List<List<Vertex>> disjointSet, Vertex u, Vertex v) {
		List<Vertex> uSubset = null, vSubset = null;
		
		for (Iterator<List<Vertex>> i = disjointSet.iterator(); i.hasNext();) {
			List<Vertex> subset = i.next();
			if (subset.contains(u)) {
				uSubset = subset;
				i.remove();
			}
			else if (subset.contains(v)) {
				vSubset = subset;
				i.remove();
			}
			
		}
		
		if (vSubset == null) {
			throw new IllegalStateException("There was a problem, Vertex v " + v.toString() + " could not be found in any subset.");
		}
		if (uSubset == null) {
			throw new IllegalStateException("There was a problem, Vertex u " + u.toString() + " could not be found in any subset.");
		}
		
		// Always merge v into u for the heck of it.
		uSubset.addAll(vSubset);
		disjointSet.add(uSubset);
		
		return disjointSet;
	}

	
}
