package emst.ds;

public class Edge {

	protected String name;
	
	protected Vertex v1;
	protected Vertex v2;
	
	protected Float weight;
	
	public Edge(String name, Vertex v1, Vertex v2) {
		this.name = name;
		this.v1 = v1;
		this.v2 = v2;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public Vertex getV1() {
		return v1;
	}
	public void setV1(Vertex v1) {
		this.v1 = v1;
	}
	public Vertex getV2() {
		return v2;
	}
	public void setV2(Vertex v2) {
		this.v2 = v2;
	}
	
	public Float getWeight() {
		return weight;
	}
	
	public void computeWeight() {
		if (v1.getCoordinates().length != v2.getCoordinates().length)
			throw new RuntimeException("v1 and v2 do not have the same dimensions.");
		
		Float sumSoFar = 0.0f;
		for (int i = 0; i < v1.getCoordinates().length; i++) {
			sumSoFar += (v1.getCoordinates()[i] - v2.getCoordinates()[i])*(v1.getCoordinates()[i] - v2.getCoordinates()[i]);
		}
		
		weight = (float)Math.sqrt(sumSoFar);
		
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Edge: ").append(name==null?"null":name).append(" v1: ").append(v1).append(" v2: ").append(v2).append("Weight: ").append(weight);
		return builder.toString();
	}
	
	public String toShortString() {
		StringBuilder b = new StringBuilder();
		b.append(getName()).append(" ").append(v1.getName()).append(" ").append(v2.getName());
		return b.toString();
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((v1 == null) ? 0 : v1.hashCode());
		result = prime * result + ((v2 == null) ? 0 : v2.hashCode());
		result = prime * result + ((weight == null) ? 0 : weight.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Edge other = (Edge) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (v1 == null) {
			if (other.v1 != null)
				return false;
		} else if (!v1.equals(other.v1))
			return false;
		if (v2 == null) {
			if (other.v2 != null)
				return false;
		} else if (!v2.equals(other.v2))
			return false;
		if (weight == null) {
			if (other.weight != null)
				return false;
		} else if (!weight.equals(other.weight))
			return false;
		return true;
	}
	
	
}
