package emst.graphics;

import javax.media.opengl.GL;

public class Camera {

	protected float maxPitchRate;
	protected float maxHeadingRate;
	protected float headingDegrees;
	protected float pitchDegrees;
	protected float forwardVelocity;
	protected float maxForwardVelocity;
	protected Quaternion heading;
	protected Quaternion pitch;
	protected Point position;
	protected Vector direction;
	
	public Camera() {
		maxPitchRate = 0.0f;
		maxHeadingRate = 0.0f;
		headingDegrees = 0.0f;
		pitchDegrees = 0.0f;
		forwardVelocity = 0.0f;
		maxForwardVelocity = 0.0f;
	}
	
	public Camera(float maxPitchRate, float maxHeadingRate,
			float headingDegrees, float pitchDegrees, float forwardVelocity,
			float maxForwardVelocity) {
		this.maxPitchRate = maxPitchRate;
		this.maxHeadingRate = maxHeadingRate;
		this.headingDegrees = headingDegrees;
		this.pitchDegrees = pitchDegrees;
		this.forwardVelocity = forwardVelocity;
		this.maxForwardVelocity = maxForwardVelocity;
		direction = new Vector(0.f, 0.f, 0.f);
		position = new Point(0.f, 0.f, 0.f);
	}

	public float getMaxPitchRate() {
		return maxPitchRate;
	}

	public void setMaxPitchRate(float maxPitchRate) {
		this.maxPitchRate = maxPitchRate;
	}

	public float getMaxHeadingRate() {
		return maxHeadingRate;
	}

	public void setMaxHeadingRate(float maxHeadingRate) {
		this.maxHeadingRate = maxHeadingRate;
	}

	public float getHeadingDegrees() {
		return headingDegrees;
	}

	public void setHeadingDegrees(float headingDegrees) {
		this.headingDegrees = headingDegrees;
	}

	public float getPitchDegrees() {
		return pitchDegrees;
	}

	public void setPitchDegrees(float pitchDegrees) {
		this.pitchDegrees = pitchDegrees;
	}

	public float getForwardVelocity() {
		return forwardVelocity;
	}

	public void setForwardVelocity(float forwardVelocity) {
		this.forwardVelocity = forwardVelocity;
	}

	public float getMaxForwardVelocity() {
		return maxForwardVelocity;
	}

	public void setMaxForwardVelocity(float maxForwardVelocity) {
		this.maxForwardVelocity = maxForwardVelocity;
	}

	public Quaternion getHeading() {
		return heading;
	}

	public void setHeading(Quaternion heading) {
		this.heading = heading;
	}

	public Quaternion getPitch() {
		return pitch;
	}

	public void setPitch(Quaternion pitch) {
		this.pitch = pitch;
	}

	public Point getPosition() {
		return position;
	}

	public void setPosition(Point position) {
		this.position = position;
	}

	public Vector getDirection() {
		return direction;
	}

	public void setDirection(Vector direction) {
		this.direction = direction;
	}

	public void setPerspective(GL gl) {
		float[] matrix;
		
		Quaternion q;
		
		// Make the Quaternions that will represent our rotations
		pitch = new Quaternion(1.0f, 0.0f, 0.0f, pitchDegrees);
		heading = new Quaternion(0.0f, 1.0f, 0.0f, headingDegrees);
		
		// Combine the pitch and heading rotations and store the results in q
		q = pitch.multiply(heading);
		matrix = q.getMatrix();
		
		// Let OpenGL set our new prespective on the world!
		gl.glLoadMatrixf(matrix, 0); // not sure about the offset.
		
		// Create a matrix from the pitch Quaternion and get the j vector 
		// for our direction.
		matrix = pitch.getMatrix();
		direction.setJ(matrix[9]);
		
		// Combine the heading and pitch rotations and make a matrix to get
		// the i and j vectors for our direction.
		q = heading.multiply(pitch);
		matrix = q.getMatrix();
		direction.i = matrix[8];
		direction.k = matrix[10];

		// Scale the direction by our speed.
		direction.scalarMultiply(forwardVelocity);		

		// Increment our position by the vector
		position.x += direction.i;
		position.y += direction.j;
		position.z += direction.k;

		// Translate to our new position.
		gl.glTranslatef(-position.x, -position.y, position.z);
		
	}
	
	public void changePitch(float degrees) {
		
	//	System.out.println(degrees);
		
		
		if (pitchDegrees + degrees < 90 || pitchDegrees + degrees > -90)
			pitchDegrees += degrees;
		else if (degrees < 0)
			pitchDegrees = -90.0f;
		else pitchDegrees = 90.0f;
		
			
	}
	
	public void changeHeading(float degrees) {
		//System.out.print(headingDegrees);
		if (headingDegrees + degrees >= 360) {
			headingDegrees = (headingDegrees + degrees - 360);
		} else {
			headingDegrees += degrees;
		}
	//	System.out.println(" " + headingDegrees);
	}
	
	public void changeVelocity(float velocity) {
	/*	if(Math.abs(velocity) < Math.abs(maxForwardVelocity))
		{
			// Our velocity is less than the max velocity increment that we 
			// defined so lets increment it.
			forwardVelocity += velocity;
		}
		else
		{
			// Our velocity is greater than the max velocity increment that
			// we defined so we can only increment our velocity by the 
			// maximum allowed value.
			if(velocity < 0)
			{
				// We are slowing down so decrement
				forwardVelocity -= -forwardVelocity;
			}
			else
			{
				// We are speeding up so increment
				forwardVelocity += maxForwardVelocity;
			}
		}*/
		forwardVelocity = velocity;
	}
}
