package emst.graphics;

/**
 * Ported to Java from http://nehe.gamedev.net/data/lessons/lesson.asp?lesson=Quaternion_Camera_Class
 * @author Ben
 *
 */
public class Quaternion {

	protected float x, y, z, w;
	
	public Quaternion() {
		x = 0.0f;
		y = 0.0f;
		z = 0.0f;
		w = 0.0f;
	}
	
	public Quaternion(float x, float y, float z, float degrees) {
		float angle = (float) ((degrees / 180f) * Math.PI);
		
		float result = (float)Math.sin( angle / 2.0f );
	
		this.w = (float)Math.cos( angle / 2.0f );

		this.x = x * result;
		this.y = y * result;
		this.z = z * result;
	}
	
	public float[] getMatrix() {
		float[] matrix = new float[16];
		
		// First row
		matrix[ 0] = 1.0f - 2.0f * ( this.y * this.y + this.z * this.z ); 
		matrix[ 1] = 2.0f * (this.x * this.y + this.z * this.w);
		matrix[ 2] = 2.0f * (this.x * this.z - this.y * this.w);
		matrix[ 3] = 0.0f;  

		// Second row
		matrix[ 4] = 2.0f * ( this.x * this.y - this.z * this.w );  
		matrix[ 5] = 1.0f - 2.0f * ( this.x * this.x + this.z * this.z ); 
		matrix[ 6] = 2.0f * (this.z * this.y + this.x * this.w );  
		matrix[ 7] = 0.0f;  

		// Third row
		matrix[ 8] = 2.0f * ( this.x * this.z + this.y * this.w );
		matrix[ 9] = 2.0f * ( this.y * this.z - this.x * this.w );
		matrix[10] = 1.0f - 2.0f * ( this.x * this.x + this.y * this.y );  
		matrix[11] = 0.0f;  

		// Fourth row
		matrix[12] = 0;  
		matrix[13] = 0;  
		matrix[14] = 0;  
		matrix[15] = 1.0f;
		
		return matrix;
	}
	
	/**
	 * this * b = c
	 * @param b
	 * @return
	 */
	public Quaternion multiply(Quaternion b) {
		Quaternion c = new Quaternion();
		
		c.w = this.w*b.w - this.x*b.x - this.y*b.y - this.z*b.z;
		c.x = this.w*b.x + this.x*b.w + this.y*b.z - this.z*b.y;
		c.y = this.w*b.y + this.y*b.w + this.z*b.x - this.x*b.z;
		c.z = this.w*b.z + this.z*b.w + this.x*b.y - this.y*b.x;
		
		return c;
	}
}
