package emst.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import emst.ds.Edge;
import emst.ds.Graph;
import emst.ds.Vertex;

public class Ms3dReader {

	
	/**
	 * Reads Milkshape 3D ascii files
	 * @param file
	 * @return
	 * @throws IOException 
	 */
	public static Graph readFile(File file) throws IOException {
		
		BufferedReader reader = new BufferedReader(new FileReader(file));
		
		List<Vertex> masterVertexList = new ArrayList<Vertex>();
		List<Edge> masterEdgeList = new ArrayList<Edge>();
		
		String line = reader.readLine();
		
		line=reader.readLine();
		line=reader.readLine();
		line=reader.readLine();
		line=reader.readLine();
		line=reader.readLine();
		
		Integer subMeshCount = Integer.parseInt(line.split(":")[1].trim());
		
		for (int j = 0; j < subMeshCount; j++) {
			line=reader.readLine();
			line=reader.readLine();
			
			// Read vertices
			int vertexCount = Integer.parseInt(line.trim());
			List<Vertex> vertices = new ArrayList<Vertex>(vertexCount + 10);
			
			for (int i = 0; i < vertexCount; i++) {
				line = reader.readLine();
				String parts[] = line.split(" ");
				Float[] coords = new Float[3];
				coords[0] = Float.parseFloat(parts[1]);
				coords[1] = Float.parseFloat(parts[2]);
				coords[2] = Float.parseFloat(parts[3]);
				
				Vertex v = new Vertex("v" + i, coords);
				vertices.add(v);			
			}
			
			// Skip the normals
			line = reader.readLine();
			int normalsCount = Integer.parseInt(line.trim());
			for (int i = 0; i < normalsCount; i++)
				line = reader.readLine();
			
			// Read the faces since we are not doing anything with normals right now, we are only concerned with the 1, 2, 3
			line = reader.readLine();
			int faceCount = Integer.parseInt(line.trim());
			int edgeCounter = 1;
			List<Edge> edges = new ArrayList<Edge>(faceCount * 3);
			for (int i = 0; i < faceCount; i++) {
				line = reader.readLine();
				String parts[] = line.split(" ");
				
				// Since they are triangles, we need 3 edges
				Edge e1 = new Edge("e" + edgeCounter, vertices.get(Integer.parseInt(parts[1])), vertices.get(Integer.parseInt(parts[2])));
				edgeCounter++;
				Edge e2 = new Edge("e" + edgeCounter, vertices.get(Integer.parseInt(parts[2])), vertices.get(Integer.parseInt(parts[3])));
				edgeCounter++;
				Edge e3 = new Edge("e" + edgeCounter, vertices.get(Integer.parseInt(parts[3])), vertices.get(Integer.parseInt(parts[1])));
				edges.add(e1);
				edges.add(e2);
				edges.add(e3);			
			}
			
			masterEdgeList.addAll(edges);
			masterVertexList.addAll(vertices);
		}
		
		Graph graph = new Graph(masterVertexList, masterEdgeList, file.getName());
		
		// attempt to remove duplicate edges
		//graph.removeDuplicateEdges();
		//graph.removeDuplicateVertices();
	//	graph.removeOrphanedEdges();
		return graph;
		
	}
}
